package uniklu.games.spaceship;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.geom.Rectangle2D;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

/**
 * Created by IntelliJ IDEA.
 * User: install
 * Date: 07.04.2008
 * Time: 16:45:13
 * To change this template use File | Settings | File Templates.
 */
public class GamePanel extends JPanel implements Runnable, KeyListener {
    private boolean running = true;
    private boolean started = false;
    private ShipSprite ship;
    private RocketSprite rocket = null;
    private ExplosionSprite xplode = null;
    private StarfieldSimulation bg;
    private DesertSimulation bgd;
    private SoundLib soundLib;
    private String infoStr;
    private static float[] blurKernel;
    private static float sigma = 2f;
    private static int kernelSize = 7;
    private int paintBackground = 0;
    private BufferedImage infoScreen = null;

    static {
        // creating the blur kernel:
        blurKernel = new float[kernelSize * kernelSize];
        for (int i = 0; i < kernelSize; i++) {
            for (int j = 0; j < kernelSize; j++) {
                blurKernel[i+j* kernelSize] = (float) (1/(2*Math.PI*sigma)*Math.exp(-(i*i+j*j)/(2*sigma*sigma)));
            }
        }
    }

    public GamePanel() {
        super();
        setDoubleBuffered(true);
    }

    public void preload() {
        // load images
        ExplosionSprite.initAnimation();
        ShipSprite.initAnimation();
        RocketSprite.initAnimation();
        // load and init parallax scrolling ...
        bg = new StarfieldSimulation();
        bgd = new DesertSimulation();
        // load and init sfx:
        soundLib = new SoundLib();
        // init sprites:
        ship = new ShipSprite();
    }

    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        // paint background:
        paintBackground(g2);

        // paint game or info screen:
        if (started) paintGame(g2);
        else paintInfo(g2);
    }

    // painting a starfield simulation ...
    private void paintBackground(Graphics2D g2) {
        g2.setColor(Color.black);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // paint stars:
        if (paintBackground==1) {
            bg.drawSprite(g2, getBounds());
        } else if (paintBackground==2) {
            bgd.drawSprite(g2, getBounds());
        }
    }

    private void paintInfo(Graphics2D graphics2D) {
        if (infoScreen == null || infoScreen.getWidth() != getWidth() || infoScreen.getHeight() != getHeight()) {
            infoScreen = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
            Graphics2D igc = infoScreen.createGraphics();
            Font myFont = Font.decode("Verdana").deriveFont(Font.BOLD, 22f);
            Font mySmallFont = Font.decode("Verdana").deriveFont(Font.BOLD, 12f);

            paintShadowedString(igc, "Sample Shooter", myFont, -32, true);
            paintShadowedString(igc, "Press 'S' to start", mySmallFont, 0, true);
            paintShadowedString(igc, "Press 'B' to activate star field", mySmallFont, 32, true);
            paintShadowedString(igc, "Press '+' or '-' to scale ship", mySmallFont, 64, true);
            igc.dispose();
        }
        graphics2D.drawImage(infoScreen, 0, 0, getWidth(), getHeight(), null);
    }

    private void paintShadowedString(Graphics2D graphics2D, String infoString, Font myFont, int offsetFromMiddle, boolean withShadow) {
        Rectangle2D bounds = null;
        if (withShadow) {
            BufferedImage binfo = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
            Graphics2D binfoG2 = binfo.createGraphics();
            binfoG2.setFont(myFont);
            bounds = binfoG2.getFontMetrics().getStringBounds(infoString, binfoG2);
            binfoG2.setColor(Color.yellow);
            binfoG2.drawString(infoString,
                getWidth() / 2 - ((int) bounds.getWidth() / 2 - 4),
                    getHeight() / 2 - ((int) bounds.getHeight() / 2) + 4 + offsetFromMiddle);
            binfoG2.dispose();
            // now blur:
            ConvolveOp op = new ConvolveOp(new Kernel(kernelSize, kernelSize, blurKernel));
            graphics2D.drawImage(binfo, op, 0, 0);
        }
        graphics2D.setFont(myFont);
        bounds = graphics2D.getFontMetrics().getStringBounds(infoString, graphics2D);
        graphics2D.setColor(Color.blue.brighter());

        graphics2D.drawString(infoString,
                getWidth() / 2 - ((int) bounds.getWidth() / 2),
                getHeight() / 2 - ((int) bounds.getHeight() / 2 )+ offsetFromMiddle);
    }

    private void paintGame(Graphics2D g2) {
        ship.drawSprite(g2);
        if (rocket != null) {
            rocket.drawSprite(g2);
            if (!rocket.isFlying()) rocket = null;
        }
        if (xplode != null) {
            xplode.drawSprite(g2);
            if (xplode.isOver()) xplode = null;
        }
    }

    public void startAll() {
        Thread t = new Thread(this);
        t.run();
    }

    public void run() {
        while (running) {
            try {
                Thread.sleep(1000 / 15);
                repaint();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private void initNewGame() {
        ship.setPosition(getWidth() / 2, getHeight() - 30);
        soundLib.startBackgroundMusic();
    }


    public void setRunning(boolean running) {
        this.running = running;
    }

    // -----------------------------------------------------------------------------------------------------------
    // ... KEY LISTENER ...
    // -----------------------------------------------------------------------------------------------------------
    public void keyTyped(KeyEvent e) {
        
    }

    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_LEFT) {
            // set movement state to the left
            ship.setMoveLeft(true);
        } else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
            ship.setMoveRight(true);
        } else if (e.getKeyCode() == KeyEvent.VK_S) {
            if (!started) {
                initNewGame();
                started = true;
            }
        }
    }

    public void keyReleased(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_LEFT) {
            // reset movement
            ship.setMoveLeft(false);
        } else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
            // reset movement
            ship.setMoveRight(false);
        } else if (e.getKeyCode() == KeyEvent.VK_SPACE) {
            if (rocket == null) {
                soundLib.sfxShot();
                rocket = ship.fire();
            } else {
                soundLib.sfxXplode();
                xplode = rocket.explode();
                rocket = null;
            }
        } else if (e.getKeyCode() == KeyEvent.VK_PLUS) {
            ship.setZoom(ship.getZoom()+0.2);
        } else if (e.getKeyCode() == KeyEvent.VK_MINUS) {
            ship.setZoom(ship.getZoom()-0.2);
        } else if (e.getKeyCode() == KeyEvent.VK_B) {
            paintBackground = ++paintBackground%3;
        } else if (e.getKeyCode() == KeyEvent.VK_J) {
            ship.jump();
        }
    }
}
